//
//  WBOBLEManager.h
//  MicroLifeDeviceSDK-Dome
//
//  Created by willy.wu on 2019/10/2.
//  Copyright © 2019 WiFi@MBP. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "MicroLifeDeviceSDK.h"
#import "MicroLifeDevice.h"
#import "MicroLifeUserInfo.h"
#import "MicroLifeDeviceInfo.h"
#import "MicroLifeDRecord.h"
#import "MicroLifeCurrentAndMData.h"
#import "MicroLifeSettingValues.h"
#import "MicroLifeCBPdataAndCalCBP.h"
#import "MicroLifeFunctionSettingValues.h"

NS_ASSUME_NONNULL_BEGIN

/**
 STATUS:

 - MicroLifeMeasurementWait: device wait countdown for next measurement
 - MicroLifeMeasurementStart: device is start BP measurement
 - MicroLifeMeasurementStop: manual press I/O to stop measurement
 */
typedef NS_ENUM(NSInteger, STATUS) {
    MicroLifeMeasurementWait    = 0x01,
    MicroLifeMeasurementStart   = 0x02,
    MicroLifeMeasurementStop    = 0x04,
};

@protocol WBODataResponseDelegate
#pragma mark - Bluetooth connection section
/**
 Mobile phone Bluetooth status changes
 
 @param state Mobile phone Bluetooth status
 */
- (void)WBOBLEManagerCellPhoneBluetoothDidUpdateState:(MicroLifeBLEState)state;

/**
 Search to device
 
 @param macAddress macAddress
 @param name name
 @param RSSI RSSI
 */
- (void)WBOBLEManagerDidDiscoverBluetoothDeviceMacAddress:(NSData *)macAddress Name:(NSString *)name RSSI:(NSNumber *)RSSI;

/**
 Device is connected
 */
- (void)WBOBLEManagerDidConnectDevice;

/**
 Device connection failed
 */
- (void)WBOBLEManagerDidFailToConnectDevice;

/**
 Disconnect
 */
- (void)WBOBLEManagerDidDisconnectDevice;

#pragma mark - Received data section
/**
 Read all history or current data from bpm

 @param data History Data
 */
- (void)WBOBLEManagerResponseReadAllHistorys:(MicroLifeDRecord *)data;

/**
Read CBP data from bpm

@param data CBP data
@param isNoData is No Data
*/
- (void)WBOBLEManagerResponseReadCBPData:(MicroLifeCBPdataAndCalCBP *)data IsNoData:(BOOL)isNoData;

/**
 Clear all history data of the BPM

 @param isSuccess Success
 */
- (void)WBOBLEManagerResponseClearHistory:(BOOL)isSuccess;

/**
 Read user ID and version data from BPM
 
 @param user user ID
 @param verData version data
 */
- (void)WBOBLEManagerResponseReadUserAndVersionData:(MicroLifeUserInfo *)user VersionData:(MicroLifeDeviceInfo *)verData;

/**
 Write a new user ID to BPM
 
 @param isSuccess Success
 */
- (void)WBOBLEManagerResponseWriteUserID:(BOOL)isSuccess;

/**
 Read Setting value
 
 @param settingValues Setting value
 */
- (void)WBOBLEManagerResponseReadSettingValue:(MicroLifeSettingValues *)settingValues;

/**
 Write BPM setting values to BPM.

 @param isSuccess Success
 */
- (void)WBOBLEManagerResponseWriteSettingValues:(BOOL)isSuccess;

/**
 Read device ID and info from BPM.
 
 @param deviceInfo device ID and info
 */
- (void)WBOBLEManagerResponseReadDeviceIDAndInfo:(MicroLifeDeviceInfo *)deviceInfo;

/**
 Read device Time from BPM

 @param deviceInfo device Time
 */
- (void)WBOBLEManagerResponseReadDeviceTime:(MicroLifeDeviceInfo *)deviceInfo;

/**
 Write device Time to BPM
 
 @param isSuccess Success
 */
- (void)WBOBLEManagerResponseWriteDeviceTime:(BOOL)isSuccess;

/**
 Read BPM function setting value from BPM

 @param functionSettingValues function setting value
 */
- (void)WBOBLEManagerResponseReadFunctionSettingValue:(MicroLifeFunctionSettingValues *)functionSettingValues;

/**
 Read BT module name from BPM
 
 @param BTModuleName BT module name
 */
- (void)WBOBLEManagerResponseReadBTModuleName:(NSString *)BTModuleName;

/**
 Start remote measurement
 D_format: only valid with CBP function.

 @param D_format only valid with CBP function
 */
- (void)WBOBLEManagerResponseStartRemoteMeasurement:(Dformat)D_format;

/**
 Remote measurement status every 5 seconds

 @param status STATUS
 @param measurementNumber Send current measurement number in auto mode.
 @param totalMeasurementNumber Send total measurement number in auto mode.
 @param countdown Send current countdown time in auto mode.
 @param totalMeasuretime Send total measurement time (seconds) in auto mode. Total measurement are count between 1st measurement to last measurement. (exclude rest time)
 */
- (void)WBOBLEManagerResponseremoteMeasurementStatusEvery5secondsWithSTATUS:(STATUS)status MeasurementNumber:(int)measurementNumber TotalMeasurementNumber:(int)totalMeasurementNumber Countdown:(int)countdown TotalMeasuretime:(int)totalMeasuretime;

/**
 Measurement results for each measurement

 @param data measurement results
 @param historyMeasuremeNumber The history measurement times store in memory.
 @param currentMeasurementTimes Send current measurement times to APP.
 @param isAverage Send Average calculation when measurement to APP.
 */
- (void)WBOBLEManagerResponseMeasurementResultsForEachMeasurement:(MicroLifeCurrentAndMData *)data HistoryMeasurementNumber:(int)historyMeasuremeNumber CurrentMeasurementTimes:(int)currentMeasurementTimes AverageCalculationWhenMeasurement:(BOOL)isAverage;
@end

@interface WBOBLEManager : NSObject
@property (nonatomic, weak) id<WBODataResponseDelegate> dataResponseDelegate;
@property (nonatomic, strong) MicroLifeDevice *bindingDevice;

@property (nonatomic, strong) NSString *result;
@property (nonatomic, strong) NSString *message;

/**
 Singleton Construction Bluetooth Controller Open equipment:
 "WatchBP Home"
 @param key Authorization key
 @return Bluetooth Controller
 */
+ (instancetype)shareInstanceWhithAuthorizationkey:(NSString *)key;

/**
 Singleton Construction Bluetooth Controller
 
 @param key Authorization key
 @param targetWBONames Set the specified display Bluetooth device name
 @return Bluetooth Controller
 */
+ (instancetype)shareInstanceWhithAuthorizationkey:(NSString *)key TargetWBONames:(NSArray *)targetWBONames;

#pragma mark - Send
/**
 Read all history or current data from bpm
 */
-(void)readAllHistorys;

/**
 Read CBP data by index from BPM
 
 @param index Data format which APP requested
 @param dformat Data format which APP requested
 */
- (void)readCBPDataWithIndex:(int)index Dformat:(Dformat)dformat;

/**
 Clear all history data of the BPM
 */
- (void)clearAllHistorys;

/**
 Disconnect the Bluetooth with BPM
 */
- (void)disconnect;

/**
 Read user ID and version data from BPM
 */
- (void)readUserAndVersionData;

/**
 Write a new user ID to BPM
 
 @param ID user ID
 */
- (void)writeUserID:(NSString *)ID;

/**
 Write BPM setting values to BPM.
 
 @param settingValues setting values
 */
- (void)writeSettingValues:(MicroLifeSettingValues *)settingValues;

/**
 Read BPM setting values from BPM
 */
- (void)readSettingValues;

/**
 Read device ID and info from BPM
 */
- (void)readDeviceIDAndInfo;

/**
 Read device Time from BPM
 */
- (void)readDeviceTime;

/**
 Write device Time to BPM
 */
- (void)writeDeviceTime;

/**
 Read BPM function setting value from BPM
 */
- (void)readFunctionSettingValue;

/**
 Read BT module name from BPM
 */
- (void)readBTModuleName;

/**
 Start remote measurement
 
 @param dformat Data format which APP requested
 */
- (void)startRemoteMeasurementWhtihCBPFunction:(Dformat)dformat;

/**
 Stop remote measurement
 */
- (void)stopRemoteMeasurement;

#pragma mark - Feature
/**
 Get the MAC address of the currently bound scale
 
 @return Mac address, if no device is bound, return nil
 */
- (NSData *)getBindingDevice;

/**
 Binding device
 
 @param macAddress Target device's Mac address
 */
- (void)bindingDevice:(NSData *)macAddress;

/**
 Untie equipment
 */
- (void)unBindingDevice;

#pragma mark - Bluetooth connection
/**
 Re-search for device
 */
- (void)reScan;

/**
 Stop searching for device
 */
- (void)stopScan;

/**
 Disconnect Bluetooth from the current device
 */
- (void)disconnectDevice;
@end

NS_ASSUME_NONNULL_END
